#!/usr/bin/env python
"""
NAME:
    mkpythonproj.py

SYNOPSIS:
    mkpythonproj.py [options] projname

DESCRIPTION:
    Make a new script or a new project, with licenses et al
    
OPTIONS:
    -h, --help             this message
    -v, --version          print version
    --author=xyz           set author to xyz
    --email=xyz            set author's email to xyz
    --url=xyz              set project's url to xyz
    --body=xyz             use body xyz [default_body]
    --pyexec=xyz           use #!xyz [/usr/bin/env python]
    --license=xyz          use license xyz [GPL]
    --script               make just a script, in current dir
    --doc                  make new document files; save old as .bak
    --setup                make new setup.py; save old as .bak
    --tests                make new pyunit-style test framework
    
EXAMPLES:
    Typically setup an alias:
       alias mkpyproj 'mkpythonproj.py --author="John Doe" \\
           email="jdoe@example.com" url="http://www.example.com" \\
           license="example_license" '

    1. Make a simple script
        mkpyproj --script myproj
       Makes a simple script:
        myproj.py

    2. Make a full project 
        mkpyproj myproj
       Makes a full project:
            AUTHORS
            COPYING
            MANIFEST.in
            README
            VERSION
            myproj/
              __init__.py
              myproj.py
            setup.py
            doc/
              go
              deshist.pdx,html
              devguide.pdx,html
              manual.prd,html
              index.pdx,html
            test/
              go              
              testdata/

    3. Make a new setup.py (needed if the setup process has changed)
        mkpyproj --setup myproj
       Makes:
          setup.py.bak   (the old one)
          setup.py       (the new one)
        
# Copyright (C) 2000,2001,2004  Harry George
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Library General Public
# License as published by the Free Software Foundation; either
# version 2 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Library General Public License for more details.
#
# You should have received a copy of the GNU Library General Public
# License along with this library; if not, write to the
# Free Software Foundation, Inc., 59 Temple Place - Suite 330,
# Boston, MA 02111-1307, USA.
#
"""

#===imports=================
import sys,os,getopt
import datetime
import string;

rundir=os.getcwd()
sys.path.insert(0,rundir)
from mkproj import templates, styles

#==globals==================
modname="mkpythonproj"
__version__="1.5"
debug_p=0

author="anonymous"
email=""
url=""
projname=""
mainclass=""
progname=""
rootdir=os.getcwd()

setup_p=0 # setup.py only
script_p=0# script only
doc_p=0   # docs only
tests_p=0 # tests only

body_name="default_body"
pyexec='/usr/bin/env python'
license_name="GPL"
license=""
license_prefix=""

now=datetime.datetime.now()
timestamp=now.strftime("%Y-%m-%dT:%H:%M:%S")
year=now.year


#===utilities===============
def msg(txt):
    sys.stdout.write(txt)
    sys.stdout.flush()
    
def debug(ftn,txt):
    if debug_p:
        sys.stdout.write("%s.%s:%s\n" % (modname,ftn,txt))
        sys.stdout.flush()

def fatal(ftn,txt):
    msg="%s.%s:FATAL:%s\n" % (modname,ftn,txt)
    raise SystemExit, msg
    
def usage():
    print __doc__

#===functions===============


#-----------------
def mkdirs():
    ftn= "mkdirs"
    os.mkdir(projname,0775)
    os.chdir(projname)
    
#-----------------
def mkauthors():
    ftn="mkauthors"
    f=open("AUTHORS","w")
    f.write(templates.AUTHORS % globals())
    f.close()

#-----------------
def mkchanges():
    f=open("CHANGES","w")
    f.write(templates.CHANGES % {'timestamp':timestamp})
    f.close()

#-----------------
def mkcopying():
    f=open("COPYING","w")
    f.write(license)
    f.close()
    
#-----------------
def mkinstall():
    ftn="mkinstall"
    f=open("INSTALL","w")
    f.write(templates.INSTALL)
    f.close()
#-----------------
def mkreadme():
    ftn="mkreadme"
    f=open("README","w")
    f.write(templates.README)
    f.close()
#-----------------
def mkversion():
    ftn="mkversion"
    f=open("VERSION","w")
    f.write("0.1\n")
    f.close()
    
#-----------------
def mkmanifest():
    ftn="mkmanifest"
    f=open("MANIFEST.in","w")
    f.write(templates.MANIFEST_IN % {'projname':projname})
    f.close()
    
#-----------------
def mktest():
    ftn = "mktest"
    currdir=os.getcwd()
    os.mkdir("test")
    os.chmod("test",0775)
    os.chdir("test")
    os.mkdir("testdata")
    os.chmod("testdata",0775)
    file=open("go","w")
    file.write("cd ..\n")
    file.write('export PYTHONPATH=../../%s:${PYTHONPATH}\n' % projname)
    file.write('PY=%s\n' % pyexec)
    file.write('${PY} ./%s/%s -d\n' % (projname,progname))
    file.close()
    os.chmod("go",0775)
    os.chdir(currdir)
#-----------------
def mktests():
    ftn = "mktests"
    currdir=os.getcwd()
    os.chdir("test")

    #---go_test---
    filename="go_test"
    file=open(filename,"w")
    file.write("#!/bin/sh\npython ./all_tests.py\n")
    file.close()
    os.chmod(filename,0775)

    #---all_tests---
    filename="all_tests.py"
    if os.path.exists(filename):
        os.rename(filename,filename+".bak")
    file=open(filename,"w")
    file.write(templates.ALL_TESTS % \
               {'progname':progname,'projname':projname,
                'mainclass':mainclass,
                'license_prefix':license_prefix,'year':year,
                'timestamp':timestamp,
                'author':author,'email':email,'url':url});
    file.close()
    os.chmod(filename,0775)

    #---basic_tests---
    filename="basic_tests.py"
    if os.path.exists(filename):
        os.rename(filename,filename+".bak")
    file=open(filename,"w")
    file.write(templates.BASIC_TESTS % \
               {'progname':progname,'projname':projname,
                'mainclass':mainclass,
                'license_prefix':license_prefix,'year':year,
                'timestamp':timestamp,
                'author':author,'email':email,'url':url});
    file.close()
    os.chmod(filename,0775)

    #---approve---
    filename="approve"
    if os.path.exists(filename):
        os.rename(filename,filename+".bak")
    file=open(filename,"w")
    file.write('cp testdata/test_$1.txt testdata/oracle_$1.txt\n')
    file.close()
    os.chmod(filename,0775)
    
    #---done---
    os.chdir(currdir)
    
#-----------------
def mkdist():
    ftn = "mkdist"
    currdir=os.getcwd()
    os.chdir("test")

    #---go_dist---
    file=open("go_dist","w")
    file.write("#!/bin/sh\n")
    file.write("cd ..\n")
    file.write("cd doc\n")
    file.write("./go\n")
    file.write("cd ..\n")
    file.write("python ./setup.py sdist\n")
    file.close()
    os.chmod("go_dist",0775)

    #---go_install---
    file=open("go_install","w")
    file.write("#!/bin/sh\n")
    file.write("cd ..\n")
    file.write("python ./setup.py build\n")
    file.write("python ./setup.py install\n")
    file.close()
    os.chmod("go_install",0775)

    #---done---
    os.chdir(currdir)

#-----------------
def mksetup():
    ftn = "mksetup"
    filename="setup.py"
    if os.path.exists(filename):
        os.rename(filename,filename+".bak")
    file=open(filename,"w")
    file.write(templates.SETUP % \
               {'projname':projname,
                'progname':progname,
                'license_prefix':license_prefix,
                'year':year,
                'author':author,
                'email':email,
                'url':url,
                'license_name':license_name,
                })
    file.close()
    os.chmod(filename,0775)

#-----------------
def mkdoc():
    ftn = "mkdoc"
    currdir=os.getcwd()
    if not os.path.exists("doc"):
        os.mkdir("doc")
    os.chdir("doc")

    filename="go"
    if os.path.exists(filename):
        os.rename(filename,filename+".bak")
    file=open(filename,"w")
    progpath=os.path.join(os.pardir,progname)
    file.write("%s -h > %s.help\n" % (progpath,projname))
    file.write("cp ../AUTHORS .\n")
    file.write("cp ../CHANGES .\n")
    file.write("cp ../INSTALL .\n")
    file.write("cp ../README .\n")

    file.write("pdx2html.py index.pdx\n")
    file.write("pdx2html.py deshist.pdx\n")
    file.write("pdx2html.py manual.pdx\n")
    file.write("pdx2html.py devguide.pdx\n")
    file.close()
    os.chmod("go",0775)

    #---index.pdx---
    filename="index.pdx"
    if os.path.exists(filename):
        os.rename(filename,filename+".bak")
    file=open(filename,"w")
    file.write(templates.INDEX_PDX % {
        'progname':progname,
        'projname':projname,
        'timestamp':timestamp,
        'author':author,
        'email':email,
        })
    file.close()


    #---default_cfg.pdx---
    filename="default_cfg.pdx"
    if os.path.exists(filename):
        os.rename(filename,filename+".bak")
    file=open(filename,"w")
    file.write(styles.DEFAULT_CFG_PDX %
               {'projname':projname,
                'timestamp':timestamp,
                'author':author,
                'email':email,
                })
    file.close()

    #---article_style.pdx---
    filename="article_style.pdx"
    if os.path.exists(filename):
        os.rename(filename,filename+".bak")
    file=open(filename,"w")
    file.write(styles.ARTICLE_STYLE_PDX);
    file.close()

    #---user manual---
    filename="manual.pdx"
    if os.path.exists(filename):
        os.rename(filename,filename+".bak")
    file=open(filename,"w")
    file.write(templates.MANUAL_PDX % {
        'projname':projname,'timestamp':timestamp})
    file.close()

    #---developer's guide---
    filename="devguide.pdx"
    if os.path.exists(filename):
        os.rename(filename,filename+".bak")
    file=open(filename,"w")
    file.write(templates.DEVGUIDE_PDX % {
        'projname':projname,
        'timestamp':timestamp,
        })
    file.close()

    #---design history---
    filename="deshist.pdx"
    if os.path.exists(filename):
        os.rename(filename,filename+".bak")
    file=open(filename,"w")
    file.write(templates.DESHIST_PDX % {
        'projname':projname,
        'timestamp':timestamp,
        })
    file.close()
    
    #---return to rootdir---
    os.chdir(currdir)
    
#-----------------------------------
def mkpackage():
    ftn="mkpackage"
    currdir=os.getcwd()
    if not os.path.exists(projname):
        os.mkdir(projname)
    os.chdir(projname)

    #---__init__.py---
    filename="__init__.py"
    if os.path.exists(filename):
        os.rename(filename,filename+".bak")
    file=open(filename,"w")
    file.write('\n')
    file.close()

    #---module---
    mkmodule()
    
    #---done---
    os.chdir(currdir)

#-----------------------------------
def mkmodule():
    ftn="mkmodule"
    file=open(progname,"w")

    #--import the body, allowing for local defs---
    currdir=os.getcwd()
    os.chdir(rundir)
    exec "from mkproj import %s as body" % body_name
    os.chdir(currdir)

    #---build the module---
    file.write(body.BODY % {\
        'pyexec':pyexec,
        'license_prefix':license_prefix,
        'author':author,'email':email,'url':url,
        'projname':projname,'progname':progname,
        'mainclass':mainclass,
        'year':year,'timestamp':timestamp})
    file.close()
    os.chmod(progname,0775)

#====================================
def main(pargs):
    ftn="main"
    global projname,progname,mainclass,\
           license,license_prefix
    if len(pargs)!=1:
        usage()
        fatal(ftn,"need 1 (and only 1) projname; pargs="+`pargs`)

    projname=pargs[0]
    mainclass=string.capitalize(projname)
    progname=projname+".py"

    cmd1="from mkproj.licenses.%s import LICENSE,LICENSE_PREFIX" % license_name
    try:
        exec cmd1
        license=LICENSE
        license_prefix=LICENSE_PREFIX
    except:
        (typ,val,trc)=sys.exc_info()
        fatal(ftn,"cannot read %s, due to %s" % (license_name,`val`))

    if script_p:
        mkmodule()
    elif setup_p:
        mksetup()
    elif doc_p:
        mkdoc()
    elif tests_p:
        mktests()
    else:
        #---full project---
        mkdirs()
        mkauthors()
        mkchanges()
        mkcopying()
        mkinstall()
        mkreadme()
        mkversion()
        mkmanifest()
        mksetup()
        mkpackage()
        mkdoc()
        mktest()
        mktests()
        mkdist()
        
#================================
if __name__ == '__main__':
    ftn="main"
    opts,pargs=getopt.getopt(sys.argv[1:],'hvd',
                 ['help','version','debug',
                  'author=','email=','url=',
                  'body=','pyexec=','license=',
                  'script','doc','setup','tests'])
    for opt in opts:
        if opt[0]=='-h' or opt[0]=='--help':
            usage()
            sys.exit(0)
        elif opt[0]=='-v' or opt[0]=='--version':
            print modname+": version="+__version__
            sys.exit(0)
        elif opt[0]=='-d' or opt[0]=='--debug':
            debug_p=1
        elif opt[0]=='--author':
            author=opt[1]
        elif opt[0]=='--email':
            email=opt[1]
        elif opt[0]=='--url':
            url=opt[1]
        elif opt[0]=='--body':
            body_name=opt[1]
        elif opt[0]=='--pyexec':
            pyexec=opt[1]
        elif opt[0]=='--license':
            license_name=opt[1]
        elif opt[0]=='--setup':
            setup_p=1
        elif opt[0]=='--doc':
            doc_p=1
        elif opt[0]=='--script':
            script_p=1
        elif opt[0]=='--tests':
            tests_p=1

    main(pargs)

#===Revision Log==============
#$Log: mkpythonproj.py,v $
#Revision 1.7  2000/04/01 17:57:06  hgg9140
#*** empty log message ***
#
#Revision 1.6  2000/04/01 17:28:17  hgg9140
#tweaked the comline options
#








